############## packages
require(mixtools)
require(Matrix)
#load MDSP_gamma function
MDSP_gamma = function (x, m.iter, eps)
{
  #x: numeric vector
  #m.iter: maximum number of iterations
  #eps: tolerance level for stopping rule
  #g.int=3*mean(x)
  
  if(length(unique(x))<2)
  {
    class.g=rep(1,length(x)); g.b=mean(x); print("homogeneous!!! in MDSP Gamma")
  }else{
    k.int=kmeans(x, centers=2)
    g.int=k.int$centers[which.max(abs(k.int$centers))]*0.95
    iter=1
    g.a=g.int; g.b=g.int+1
    while (abs(g.b-g.a)>eps & iter < m.iter)
    {
      class.g=as.numeric(abs(x-g.a)< abs(x))
      g.b=sum(class.g*x)/sum(class.g)
      iter=iter+1
      g.a=g.b
    }
    if(iter==m.iter) {print("WARNING: maxium iteration achieves")}
  }
  return(list(l.group=class.g, gamma=g.b))
}

#l.group: group indicator (non-zero subgroup =1) 
# gamma: sub-homogeneous effect of non-zero subgroup



### Modified BIC function


#########################################

MDSP_l=function(Y,X,Z,m,l1,kp,beta.int=NULL)
{
  #############################################
  #############  ADMMM Algorithm ##############
  #############################################
  
  ## Input data X (heterogeneous, nm by p.b), 
  ## Z (homogeneous including intercept, nm by p.a),  
  ## and Y (nm by 1): by default, the longitudinal id is rep(1:n, each=m)
  ## m: number of individual measurements 
  ## l1: shrinkage parameter
  ## kp: ADMM step
  ## beta.int: initial of individualized parameters (np.b by 1) 
  
  n=length(Y)/m
  p.b=dim(X)[2]
  p.a=dim(Z)[2]
  X.td=cbind(Z, t(KhatriRao(diag(n)%x%matrix(1,1,m), t(X))))
  
  ### set tunning hyperparameters and initial values
  ## hyper para
  kpp=kp #ADMM step
  lm.s=l1 #L1 regularization
  
  ## initials
  if (is.null(beta.int)){nu.int=rep(0,n*p.b)}else{nu.int=beta.int}
  lm.int=rep(0,n*p.b)
  
  lm.h.new=lm.int
  nu.h.new=nu.int
  
  a.h.new=rep(0,p.a)
  b.h.new=rep(0,n*p.b)
  
  theta.h=rep(0,n*p.b+p.a)
  theta.h.new=rep(1,n*p.b+p.a)
  
  iter=1
  max.iter=100
  eps=0.001
  
  error=rep(1,max.iter-1)
  
  
  #NOTE: It's better to have a warm start
  
  ptm <- proc.time()
  
  ##### Main loop
  while(iter < max.iter & sqrt(mean((theta.h-theta.h.new)^2)) > eps)
  {
    error[iter]=sqrt(mean((theta.h-theta.h.new)^2))
    
    ###
    lm.h=lm.h.new
    nu.h=nu.h.new
    
    a.h=a.h.new
    b.h=b.h.new
    theta.h=theta.h.new
    
    ############## theta updates#################
    theta.h.new=solve(t(X.td)%*%X.td+bdiag(list(matrix(0,p.a,p.a),kpp*diag(n*p.b))))%*%(t(X.td)%*%Y+c(rep(0,p.a),kpp*nu.h-lm.h))
    a.h.new=theta.h.new[1:p.a]
    b.h.new=theta.h.new[-(1:p.a)]
    
    ######### nu,gamma updates ###############
    B.h.new=matrix(b.h.new,p.b,n)
    Nu.h.new=matrix(nu.h,p.b,n)
    g.h=rep(0,p.b)
    g.h.new=rep(0,p.b)
    cl.h=matrix(0,p.b,n)
    Lm=matrix(lm.h,p.b,n)
    
    for (k in 1:p.b)
    {
      sp.0=MDSP_gamma(B.h.new[k,], 50,0.001)
      g.h[k]=sp.0$gamma-0.01
      g.h.new[k]=g.h[k]+0.01
      cl.h[k,]=sp.0$l.group
    }
    
    iter.nu=1
    ### find gamma, nu
    for (k in 1:p.b)
    {
      b.t.new=B.h.new[k,]+kpp^{-1}*Lm[k,]
      while (iter.nu <50 & abs(g.h[k]-g.h.new[k])>0.001)
      {
        g.h[k]=g.h.new[k]  
        #lm.s.ad=lm.s*min(iter/5,1)
        lm.s.ad=lm.s*m
        #lm.s.ad=lm.s
        nu.0=sign(b.t.new)*as.numeric(abs(b.t.new)>lm.s.ad/kpp)*(abs(b.t.new)-lm.s.ad/kpp)
        nu.g=g.h[k]+sign(b.t.new-g.h[k])*as.numeric(abs(b.t.new-g.h[k])>lm.s.ad/kpp)*(abs(b.t.new-g.h[k])-lm.s.ad/kpp)
        Nu.h.new[k,]=nu.g*cl.h[k,]+nu.0*(1-cl.h[k,])
        
        sp.h=MDSP_gamma(Nu.h.new[k,], 50,0.001)
        g.h.new[k]=sp.h$gamma
        cl.h[k,]=sp.h$l.group
        iter.nu=iter.nu+1
        if(iter.nu==50) {print("WARNING: maxium iteration (nu) achieves")}
      }
    }
    
    ### lambda updates
    nu.h.new=as.vector(Nu.h.new)
    lm.h.new=lm.h+kpp*(b.h.new-nu.h.new)
    
    iter=iter+1
    if(iter==max.iter) {print("WARNING: maxium iteration achieves")}
  }
  
  return(list(alpha=a.h.new, Beta=B.h.new, Nu=Nu.h.new,gamma.h=g.h.new, group=cl.h, it.errors=error[1:(iter-1)]))
  
  print(proc.time()-ptm)
  
}
#################################################################################


# #############Data generation
n=40
m=20
p.x=1
p.z=3

set.seed(1122)
### parameter
gamma=1
id.beta=rep(c(1,0), each=n/2)
beta=gamma*id.beta
alpha=rep(1,p.z)


  ### data
  X=matrix(rnorm(n*m*p.x),n*m,p.x)
  Z=cbind(1, matrix(rnorm(n*m*(p.z-1)),n*m,(p.z-1)))
  X.td=cbind(Z, t(KhatriRao(diag(n)%x%matrix(1,1,m), t(X))))
  
  Y=X.td%*%c(alpha,beta)+rnorm(n*m)
  
  
  ### model
  fit.m=MDSP_l(Y,X,Z,m=20,l1=0.1,kp=1)
  fit.m1=MDSP_l(Y,X,Z,m=20,l1=0.2,kp=1,beta.int=round(as.vector(fit.m$Beta),2))
  fit.m2=MDSP_l(Y,X,Z,m=20,l1=0.5,kp=5,beta.int=round(as.vector(fit.m$Beta),2))
  
 


###
par(mfrow=c(1,3))
plot(fit.m$it.errors)
plot(fit.m1$it.errors)
plot(fit.m2$it.errors)

par(mfrow=c(2,3))
plot(c(fit.m$Nu))
plot(c(fit.m1$Nu))
plot(c(fit.m2$Nu))

plot(c(fit.m$Beta))
plot(c(fit.m1$Beta))
plot(c(fit.m2$Beta))
# 




